package it.openutils.spring.rmibernate.server.aspects;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.hibernate.SessionFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Main server interceptor on rmi calls
 * @author mmolaschi
 * @version $Id: $
 */
public class HibernateLazyRmiInterceptor implements MethodInterceptor
{

    /**
     * log
     */
    protected static final Logger logger = LoggerFactory.getLogger(HibernateLazyRmiInterceptor.class);

    private SessionFactory sessionFactory;

    /**
     * Contructor
     * @param sessionFactory hibernate session factory
     */
    public HibernateLazyRmiInterceptor(SessionFactory sessionFactory)
    {
        this.sessionFactory = sessionFactory;
    }

    /**
     * {@inheritDoc}
     */
    public Object invoke(MethodInvocation invocation) throws Throwable
    {
        try
        {
            Object retVal = invocation.proceed();

            SerializationInterceptor.clean();
            // Proxy class for serialization work
            return SerializationInterceptor.getEnhancedObject(retVal, sessionFactory);
        }
        catch (Throwable ex)
        {
            if (logger.isInfoEnabled())
            {
                logger.info("Processing of " + " remote call resulted in exception: ", ex);
            }

            throw ex;
        }
    }

}
